function [ux,uz,ax,az,z,pot]=linkinematic(A,w,phase,surface,x,t,nz,d,method)
%Calculate the linear kinematic from the given wave amplitude and its
    %frequency component
%A=wave amplitude
%w=frequency of each A
%d=water depth
%x=x coordinate
%nz=number of vertical coordinates from depth to  surface
%t=time
%u=water particle velocity, column 1=x, column 2=z
%a=wave particle acceleration, column 1=x, column 2=z
%p=pressure
%method=method of calculating kinematic for z>0:
    %1.Wheeler Stretching
    %2.Stansberg Linear Extrapolation
%In this function,the vectorization to gain faster computation only done
    %for linear extrapolation method (stansberg method)

if ~exist('method','var'); method=1; end
if ~exist('d','var'); d=1000; end
if ~exist('x','var'); x=0; end
if ~exist('t','var'); t=0; end
if ~exist('nz','var'); nz=50; end

%concentrate the points near the surface

z=[-d -80 -70 -60:5:-30 -29:30];

pot=zeros(length(x),length(t),length(z));
ux=zeros(length(x),length(t),length(z));
uz=zeros(length(x),length(t),length(z));
ax=zeros(length(x),length(t),length(z));
az=zeros(length(x),length(t),length(z));

k=wavenum(w,d);
switch method
case 1
    lent=length(t);lenz=length(z);lenx=length(x);
parfor it=1:lent
    for iz=1:lenz;%nz
        for ix=1:lenx
            if z(iz)>=surface(ix,it)
                pot(ix,it,iz)=0;
                ux(ix,it,iz)=0;
                uz(ix,it,iz)=0;
                ax(ix,it,iz)=0;
                az(ix,it,iz)=0;
            else
                [pot(ix,it,iz),ux(ix,it,iz),uz(ix,it,iz),ax(ix,it,iz),...
                    az(ix,it,iz)]=...
                    wheeler(A,k,w,phase,surface(ix,it),x(ix),z(iz),t(it),d);
            end
        end
    end
end

case 2
lent=length(t);lenx=length(x);
lenz=[1 1 length(z)];
        
g=9.812;%gravity acceleration

pota=g*A./(w.*cosh(k*d));
ua=g*k.*A./(w.*cosh(k*d));
aa=g*k.*A./cosh(k*d);
        
UXderA=g*k.^2.*A.*sinh(k*(d+0))./(w.*cosh(k*d));
AXderA=g*k.^2.*A.*sinh(k*(d+0))./cosh(k*d);
POTderA=(g*k.*A./w).*sinh(k*(0+d))./cosh(k*d);
UZderA=(g*k.*A./w)./cosh(k*d);
AZderA=g*k.*A./cosh(k*d);
        
%Vectorization for efficient and faster calculation
K=repmat(k,lenz);W=repmat(w,lenz);PHASE=repmat(phase,lenz);
ZK=repmat(permute((d+min(z,0)),[1 3 2]),[size(k) 1]);
        
potai=repmat(pota,lenz).*cosh(K.*ZK);
uxa=repmat(ua,lenz).*cosh(K.*ZK);
uza=repmat(ua,lenz).*sinh(K.*ZK);
axa=repmat(aa,lenz).*cosh(K.*ZK);
aza=repmat(aa,lenz).*sinh(K.*ZK);
        
ZD=repmat(permute(max(z,0),[1 3 2]),[size(k) 1]);
SCALEZ=repmat(permute(max(z,0)./z,[1 3 2]),[size(k) 1]);
UXderAI=repmat(UXderA,lenz).*ZD;
AXderAI=repmat(AXderA,lenz).*ZD;
POTderAI=repmat(POTderA,lenz).*ZD;
UZderAI=repmat(UZderA,lenz).*...
    ((repmat(sinh(k*(0+d)),lenz).*SCALEZ)+(ZD.*repmat(k.*cosh(k*(0+d)),lenz)));
AZderAI=repmat(AZderA,lenz).*...
    ((repmat(sinh(k*(0+d)),lenz).*SCALEZ)+(ZD.*repmat(k.*cosh(k*(0+d)),lenz)));

parfor it=1:lent
    for ix=1:lenx
        [pot(ix,it,:),ux(ix,it,:),uz(ix,it,:),ax(ix,it,:),az(ix,it,:)]=...
            stansberg(K,W,PHASE,x(ix),t(it),potai,uxa,uza,axa,aza,...
            UXderAI,AXderAI,POTderAI,UZderAI,AZderAI);
    end
end
%showing value only for z below water surface elevation
Zz=repmat(permute(z,[1 3 2]),[length(x) length(t) 1]);
activate=Zz<repmat(surface,[1 1 length(z)]);
pot=pot.*activate;
ux=ux.*activate;
uz=uz.*activate;
ax=ax.*activate;
az=az.*activate;

case 3
lent=length(t);lenx=length(x);
lenz=[1 1 length(z)];
        
g=9.812;%gravity acceleration

pota=g*A./(w.*cosh(k*d));
ua=g*k.*A./(w.*cosh(k*d));
aa=g*k.*A./cosh(k*d);
               
%Vectorization for efficient and faster calculation
K=repmat(k,lenz);W=repmat(w,lenz);PHASE=repmat(phase,lenz);
ZK=repmat(permute((d+min(z,0)),[1 3 2]),[size(k) 1]);
        
potai=repmat(pota,lenz).*cosh(K.*ZK);
uxa=repmat(ua,lenz).*cosh(K.*ZK);
uza=repmat(ua,lenz).*sinh(K.*ZK);
axa=repmat(aa,lenz).*cosh(K.*ZK);
aza=repmat(aa,lenz).*sinh(K.*ZK);

parfor it=1:lent
    for ix=1:lenx
        [pot(ix,it,:),ux(ix,it,:),uz(ix,it,:),ax(ix,it,:),az(ix,it,:)]=...
            constantstrech(K,W,PHASE,x(ix),t(it),potai,uxa,uza,axa,aza);
    end
end
%showing value only for z below water surface elevation
Zz=repmat(permute(z,[1 3 2]),[length(x) length(t) 1]);
activate=Zz<=repmat(surface,[1 1 length(z)]);
pot=pot.*activate;
ux=ux.*activate;
uz=uz.*activate;
ax=ax.*activate;
az=az.*activate;
end